@@ -85,9 +85,7 @@ void UseBuilder::visitClassExtends(ClassExtendsAst *node)
 
 void UseBuilder::visitExpr(ExprAst* node)
 {
-    UseExpressionVisitor v(m_editor, this);
-    node->ducontext = currentContext();
-    v.visitNode(node);
+    visitNodeWithExprVisitor(node);
 }
 
 void UseBuilder::visitGlobalVar(GlobalVarAst* node)
@@ -103,25 +101,26 @@ void UseBuilder::visitGlobalVar(GlobalVarAst* node)
 void UseBuilder::visitStaticScalar(StaticScalarAst* node)
 {
     if (currentContext()->type() == DUContext::Class) {
-        UseExpressionVisitor v(m_editor, this);
-        node->ducontext = currentContext();
-        v.visitNode(node);
+        visitNodeWithExprVisitor(node);
     }
 }
 
 void UseBuilder::visitStatement(StatementAst *node)
 {
-    AstNode* visitNode = 0;
     if (node->foreachVar) {
-        visitNode = node->foreachVar;
+        visitNodeWithExprVisitor(node->foreachVar);
     } else if (node->unsetVariablesSequence) {
-        visitNode = node;
+        visitNodeWithExprVisitor(node);
     }
 
-    if (visitNode) {
-        UseExpressionVisitor v(m_editor, this);
-        visitNode->ducontext = currentContext();
-        v.visitNode(visitNode);
+    if (node->foreachExprAsVar) {
+        visitNodeWithExprVisitor(node->foreachExprAsVar);
+    }
+    if (node->foreachVarAsVar) {
+        visitNodeWithExprVisitor(node->foreachVarAsVar);
+    }
+    if (node->foreachVariable) {
+        visitNodeWithExprVisitor(node->foreachVariable);
     }
 
     UseBuilderBase::visitStatement(node);
@@ -130,8 +129,7 @@ void UseBuilder::visitStatement(StatementAst *node)
 void UseBuilder::visitCatchItem(CatchItemAst *node)
 {
     if (node->catchClass) {
-        DeclarationPointer dec = findDeclarationImport(ClassDeclarationType, node->catchClass);
-        newCheckedUse(node->catchClass, dec);
+        buildNamespaceUses(node->catchClass, ClassDeclarationType);
     }
     UseBuilderBase::visitCatchItem(node);
 
@@ -168,7 +166,7 @@ void UseBuilder::visitUseNamespace(UseNamespaceAst* node)
 
 void UseBuilder::buildNamespaceUses(NamespacedIdentifierAst* node, DeclarationType lastType)
 {
-    QualifiedIdentifier identifier = identifierForNamespace(node, m_editor);
+    const QualifiedIdentifier identifier = identifierForNamespace(node, m_editor);
     QualifiedIdentifier curId;
     curId.setExplicitlyGlobal(identifier.explicitlyGlobal());
     Q_ASSERT(identifier.count() == node->namespaceNameSequence->count());
@@ -196,5 +194,15 @@ void UseBuilder::openNamespace(NamespaceDeclarationStatementAst* parent, Identif
     UseBuilderBase::openNamespace(parent, node, identifier, range);
 }
 
+void UseBuilder::visitNodeWithExprVisitor(AstNode* node)
+{
+    UseExpressionVisitor v(m_editor, this);
+    node->ducontext = currentContext();
+    v.visitNode(node);
+
+    if (v.result().hadUnresolvedIdentifiers()) {
+        m_hadUnresolvedIdentifiers = true;
+    }
+}
 
 }
